from PIL import Image
from PIL import ImageOps
import moonmask.mask as mask
import moonmask.custom_image as custom_image
import moonmask.moon_image as moon
from moonmask.res.constants import *

class Collage():

    def __init__(self):
        self.main_image = None
        self.moon_mask_negative_space = None
        self.moon_mask_positive_space = None
        self.moon_mask = None
        self.img_size = (1000,1000)
        self.image_queue = {}
        return

    def set_mask(self, date=None, relative_date="today", filename=None):
        """Sets the image that will be used as a mask on the image

        Keyword arguments:
        date -- the date in format YYYYMMDD
        relative_date -- defaults to "today". Other accepted options are "yesterday" and "tomorrow"
        """
        self.moon_image = moon.MoonImage(size=self.img_size)
        self.moon_mask = self.moon_image.set_moon_image(date=date, relative_date=relative_date, filename=filename)
        return

    def add_to_image_queue(self, image_key, image=None, url="", instagram_url="", filename="", color=""):
        if image:
            self.image_queue[image_key] = image
        else:
            self.image_queue[image_key] = custom_image.CustomImage(self.img_size).set_image(url, instagram_url, filename, color)
        return

    def list_image_queue(self):
        for image_key in self.image_queue:
            print(image_key)
        return

    def get_moon_phase_date(self):
        return self.moon_image.get_moon_phase_date()

    def set_main_image(self, image_key, url="", instagram_url="", filename="", color=""):
        self.main_image = self.create_image(image_key, url, instagram_url, filename, color)
        return

    def set_mask_negative_space(self, image_key, url="", instagram_url="", filename="", color=""):
        self.moon_mask_negative_space = self.create_image(image_key, url, instagram_url, filename, color)
        return

    def set_mask_positive_space(self, image_key, url="", instagram_url="", filename="", color=""):
        self.moon_mask_positive_space = self.create_image(image_key, url, instagram_url, filename, color)
        return

    def create_image(self, image_key, url="", instagram_url="", filename="", color=""):
        if not image_key in self.image_queue:
            if not url and not instagram_url and not filename and not color:
                raise Exception("There is no image in queue that has the key " + image_key)
            else:
                created_image = custom_image.CustomImage(self.img_size).set_image(url, instagram_url, filename, color)
                self.image_queue[image_key] = created_image
        else:
            created_image = self.image_queue[image_key]
        return created_image

    def make_collage(self, filename, main_image_file=None, mask_file=None, positive_space_file=None, negative_space_file=None, positive_space_transparency=200, negative_space_transparency=50, dimensionality=3, img_size=1000):
        img_size = (img_size, img_size)

        if self.main_image:
            main_image = self.main_image#custom_image.CustomImage(img_size).load_from_instagram(test_url)
            mask_shaped_main_image = self.main_image#custom_image.CustomImage(img_size).load_from_filesystem(main_image_file)
        else:
            main_image = Image.new('RGB', img_size, 'black')
            mask_shaped_main_image = Image.new('RGB', img_size, 'black')

        #give the main_image image the requested white borders
        white_negative_space = Image.new('RGB', img_size, "white")

        main_image = ImageOps.fit(main_image, img_size, Image.ANTIALIAS)
        mask_shaped_main_image = ImageOps.fit(mask_shaped_main_image, img_size, Image.ANTIALIAS)

        if self.moon_mask_negative_space:
            negative_space = self.moon_mask_negative_space#Image.open(negative_space_file, 'r')
            negative_space = ImageOps.fit(negative_space, img_size, Image.ANTIALIAS)
        else:
            negative_space= Image.new('RGB',img_size,'black')

        if self.moon_mask_positive_space:
            mask_shaped_positive_space = self.moon_mask_positive_space#Image.open(positive_space_file)
            mask_shaped_positive_space = ImageOps.fit(mask_shaped_positive_space, img_size, Image.ANTIALIAS)
        else:
            mask_shaped_positive_space = Image.new('RGB', img_size, 'white')

        negative_space_mask = mask.Mask(negative_space, 255, img_size, negative_space_transparency).get_mask()

        #put transparent negative_space over main_image
        main_image.paste(negative_space, (0,0), mask=negative_space_mask)

        transparency_divisor = dimensionality + 1

        #loop through a range, so that different brightnesses of the mask
        #are represented with different transparencies of the positive_space
        for i in range(1, dimensionality + 1):
            transparency_divisor -= 1

            current_transparency = (positive_space_transparency / transparency_divisor) + ((255 - positive_space_transparency / transparency_divisor) / transparency_divisor)
            #255 / 1 + (255 - 255) / 1 = 255
            #255 / 2 + (255 - 123) / 2 = 123
            #255 / 3 + (255 - 61 / 3) = 61

            current_mask = mask.Mask(self.moon_mask, 40 * i, img_size).get_mask()
            mask_transparent = mask.Mask(self.moon_mask, 40 * i, img_size).get_mask()
            mask_transparent = mask_transparent.point(lambda j: min(j * 25, current_transparency))

            mask_shaped_positive_space.putalpha(mask_transparent)

            #reput mask shaped main_image on top of transparent negative_space
            mask_shaped_main_image.putalpha(current_mask)
            main_image.paste(mask_shaped_main_image, (0,0), mask=mask_shaped_main_image)

            #put positive_space over mask
            main_image.paste(mask_shaped_positive_space, (0,0), mask=mask_transparent)

        main_image.save(filename + ".jpg", format='JPEG')
        main_image.show()



import moonmask

class UI:

    def walk_through(self):
        print("Hey welcome, this is a little command line version of a tool called moonmask that you can use to get the current phase of the moon and use it to make a collage with other images. Press enter to continue.")
        input()
        i = input("You have a couple of options.\n\nPlease choose one of the following:\n\n(1) Get the current moonphase and open up an image of it on your computer\n\n(2) Get the current moonphase and use it to make a collage with some other images")
        if i == "1":
            m = moonmask.MoonMask()

            m.collage.set_mask()
            m.save_collage()
            #m.post_to_instagram("sadiie_parker", "Dr0w**ap!!", save_session=True)
        elif i == "2":
            input("OK, since you want to make a collage, let's get some information together.")

#ui = UI()
#ui.walk_through()

m= moonmask.MoonMask()
m.set_moon_phase()
# girl with red jacket m.set_positive_space(instagram_url="https://www.instagram.com/p/BvJV1GnBvPD/")
m.set_main_image(instagram_url="https://www.instagram.com/p/BgRtiOplAXq/")
print("https://www.instagram.com/p/BgRtiOplAXq/")
m.set_positive_space(filename="moonmask.jpg")
#m.set_main_image(url="https://lh3.googleusercontent.com/G4Dks-VYZrokqTE2z-gznMEIANmjYbiIhspCh8pn4bvW_4sjTVfllo8xyeikW5gykEIt6JzHan3piuvg5ZkkPG75Cv6YJ5T0Ue0ByvrhZJyKl8xmPjZWVRsEYRPxegcjR-Dtb_ioHwUL2Wpt0PXdHqEVPEBu0vvvmRn4BVSWX8mYffTdsWNaeSycHNDbspz-5439rIeWsrEjfnETEFeHwT7eiau1tB4tOaUGCODNzz78oQBAkVtN8u7eo-Ztgpnj7voSZCFDvNnkQAfXLpP2x7o9atZbCsz8ATi_UpTXPJqb9fP9mSW5WSkQ9aHJpWoAULX14eKKHm3n7ZXht-b1-yvQ2qjKlU3zKASam4P-e99ZBRL8BDi5koiQ2tqVDhxRKWqdqUBUfHEFsREj_Znd05Skh6CPuZM7bTSAmSzic82kVxB4zylyb7sphHy39Rz0QOvuaF8tZET5C_BBhIfiLvk5QYVg0A_zWmIBtkcFmD9Dw4wBWpcabtyxavSsmyfPoEFOsnHG3J5KNWCOqNjQK8VcGpNz7IV10-nlM28hO8ruGIKNjkQUvgqU4fKeSpo1SYeiPykAocWxuw4M-VysbTNNoBxjjLjMHqtd7d81y1eCQY50JfnTmtBngmjgP0PTgSjzQQy99ZnadDVA7ZinA-s9g5gCgcfr=w864-h648-no")
#m.set_negative_space(color="white")
# windmills m.set_main_image(url="https://lh3.googleusercontent.com/CQPY4K212wMVrnKzV0VP8eNxnkyYkmlL5liO6vKG3HauG4TbIRM9cl4PWnY1gEE6BFnpBbGYAy6OZD9fLOv8yPSkD815rgsmosuWKpQX59F8eENDxW2hd-Z-B4oXqCa2HC-7cimSNs0Qw_KCIJbsq7opQIqdT5USpcGVPdNmXwaVhOt1a9fgF-RyfowmrSBSfvpEhoB3eBrIyHL7Byw4ZwRA58EAjUFUIhw-DEVICH2vF0uJbR2spu1RHBailSWc-DFghAwAtG3QkHxM6USIdA8KBowBQrXiaVJ54JMmp3NpBKUIYlNDrL-VSQWylJFRLfQM7t28UzMvT7k9fHyIOFIZT5hs0NeUrBHFkw5TnDP625LBu-Ebh-zH72VxdgS0G5bkQ1C9X6QuasqgRwjlOsvqWwRTq7jO_TzfFU3wCLTnqXzhXyC6kO7hvy33etj2Xo02ik2kJhUFq8WA7Zy0xQs5Q1h2EqeQB8oye00KU-TqzKdi4PNFkYlARhJsT7JSU2tBwqrLt_Evxj-bMyYJd48ixP9clo3C7GOjnvkAzlolWE-sCeK8F1wbSj5SvHp14R097Ww9tSjsFEXnDVVsUaqEK38H7QNhsniEYV1D-TIgKj8HEYmwDSqlPccMQ6vleKjWBaMr71ruuRbtOQ6OD7G9Bb-FFtlH=w487-h648-no")
m.set_negative_space(color="white")
m.save_collage(negative_space_transparency=200, positive_space_transparency=255, dimensionality=1)
m.post_to_instagram("mun_fases","Dr0w**ap!!",caption="waning gibbous moon\n*\n*\n*\n*#moonphases #waninggibbous #moon #moonart #shadow #shadowselfie #selfreflection #lunar #lunart #sagittarius #sagittariusmoon #cycles #cyclical #astrology #digitalcollage #pythonimaginglibrary #python")



# UI TEXT STRINGS -- TODO WRITE FOR DIFFERENT LANGUAGES
# TODO - organize these so they aren't random lists / strings, and instead consolidated in some dictionaries or something

PURPOSES_OF_PACKAGE_TUTORIAL_STEP = [
    "Let's talk about what we can do with this package.",
    "* We can get a 'mask' of the current moon phase",
    "* We can use this mask to create an image of the current moon phase.",
    "* We can save this image as is.",
    "* We can get custom images from outside sources, like urls, files, or instagram posts.",
    "* We can use these images as masks together with the mask we got from the moonphase, and create collages."
]
IMPORT_CLASS_TUTORIAL_STEP = [
    "To use this package, it needs to be imported into wherever we're running it. Considering you're already running this tutorial, you've probably already imported it in one way or another.",
    "So, the first line in your code should read:",
     "{}",
    "This import statement imports a class called MoonMaskUI, which is what we'll use to interact with the package.",
]

INITIALIZE_UI_TUTORIAL_STEP = [
    "Now that we have the class MoonMaskUI imported, let's use it. We do this like this:",
    "{}",
    "This code makes a new variable, which we're choosing to call 'ui', and sets it equal to an instance of our class.",
    "Now, we will be able to use this variable 'ui' to make an image."
]

SET_MOON_PHASE_TUTORIAL_STEP = [
    "Since we're here to get the moonphase, let's start there. To get the current moonphase, we call a method 'set_moon_phase' on our ui.",
    "{}",
    "This method will ask for current moon phase from the nasa repository of images, and use it to create a mask, which will later be used to make an image.",
]


INTRO_SAVE_COLLAGE_TUTORIAL_STEP = ["Now that we've set the moonphase mask, let's see what it looks like when transformed to an image.",
                        "We will try this command:",
                        "{}",
                        "This method will make an image and (depending on how we're running this script) show it to us."
                        ]

SET_POSITIVE_SPACE_TUTORIAL_STEP = [
    "Now, let's try customizing this moonphase image a bit. Let's change the positive space in the moon.",
    "Take a look at the little sketch above for what part we are considering the 'positive space' in an image of a full moon.",
    "Now, we change this positive space to having a yellow color like this:",
    "{}"
]

SET_POSITIVE_SPACE_COMMAND = "ui.set_positive_space(color='yellow')"

#TODO set command_prompt icons for these command2-4 in the way you're setting the others
SET_POSITIVE_SPACE_COMMAND2 = ">>  ui.set_positive_space(url='https://upload.wikimedia.org/wikipedia/en/2/27/Bliss_%28Windows_XP%29.png')"
SET_POSITIVE_SPACE_COMMAND3 = ">>  ui.set_positive_space(instagram_url='https://www.instagram.com/p/Bjo2ftsH2KC/')"
SET_POSITIVE_SPACE_COMMAND4 = ">>  ui.set_positive_space(filename='i_am_not_a_real_file.jpg')"

LETS_SAVE_TUTORIAL_STEP = [
    "Let's save this to take a look at it.",
    "{}"
]

SET_NEGATIVE_SPACE_TUTORIAL_STEP = [
    "Now let's change the negative space to yellow with this command:",
    "Take a look at the little sketch above for what part we are considering the 'negative space' in an image of a full moon.",
    "{}"
]

SET_MAIN_IMAGE_TUTORIAL_STEP = [
    "Finally, let's change the 'main image' here. The 'main image' is what these masks are being 'pasted' onto.",
    "We change the main image like this:",
    "{}",
    "We are using a different way of declaring the color here -- it's in 'RGB' format."
]

MORE_INFO = [
    "In this tutorial, we only changed the colors of the positive space, negative space, and main image.",
    "We used the keyword argument 'color' to do this, like ",
    "{}",
    "To use images instead of colors, we can use different keyword arguments, like 'url', 'instagram_url', and 'filename'.",
    "For example:",
    SET_POSITIVE_SPACE_COMMAND2,
    SET_POSITIVE_SPACE_COMMAND3,
    SET_POSITIVE_SPACE_COMMAND4,
    "If you'd like to play around with these different keyword arguments, please give it a try outside of this tutorial!"
]

SET_NEGATIVE_SPACE_COMMAND = "ui.set_negative_space(color='yellow')"

SET_MAIN_IMAGE_COMMAND = "ui.set_main_image(color=(240, 128, 128))"

SAVE_COLLAGE_COMMAND = "ui.save_collage(filename='moon')"

TUTORIAL_PROMPT_FOR_COMMAND = "Type out the command from above (the command after '>>>'. For this tutorial, you'll have to exactly match it. If you get stuck and need to exit, remember that ctrl-c will get you out of this tutorial."

TUTORIAL_TRY_AGAIN_PROMPT = "That doesn't match '{}', please try again:"

SET_MOON_PHASE_COMMAND = "ui.set_moon_phase()"

IMPORT_UI_COMMAND = "from moonmask.moonmask_ui import MoonMaskUI"

INITIALIZE_UI_COMMAND = "ui = MoonMaskUI()"

THIS_IS_A_TUTORIAL_MESSAGE = "This is a small tutorial on how to use it. For more info on this package, please head over to https://github.com/spacerest/moonmask/README.md"

FINISHED_TUTORIAL_MESSAGE = "Well, this is the end of the moonmask tutorial! For more info on this package, please head over to https://github.com/spacerest/moonmask/README.md. If anything isn't working, please submit an issue on github. Thanks! :)"

NAVIGATION_REMINDER = "To exit this tutorial you can press ctrl-C at any time. To go back in the tutorial you can scroll up ^."

USER_SUCCESS_MESSAGE = "Nice :)"

CREDITS_LIST = ["Thanks to Ernie Wright of Nasa for creating the moon visualizations that the generated moon masks are based on. Check out his visualizations here: https://svs.gsfc.nasa.gov/cgi-bin/search.cgi?person=105 ","Thanks to Joan Stark for the ascii art used in this tutorial. I read a nice interview by her here: http://www.lastplace.com/ASCIIart/starkascii.htm.", "If you'd like to help on this project and get added to this credits list, please contact me <3 (my github username is spacerest)"]

SHOW_INFO = "Welcome to moonmask! You can use this package to get a 'mask' of the moonphase for a particular day. You can use this mask to create an image of the current moon phase, or to make a collage using your own photos."

PRESS_ENTER_TO_CONTINUE = "(Please press enter to continue...)"

SAVE_COLLAGE_CONFIRMATION = "If you're running this tutorial locally, a sample image of your collage should have opened up just now. If you're running this tutorial on an online repl like repl.it, then you won't see any image until the end of the tutorial. Anyway, if an image opened up, it is a temporary file that has just opened up to show you what the current image looks like. There should also be a saved version of the image by this name: "

OVERWRITE_FILENAME_NOTICE = "Since you haven't specified a different filename, it will be saved as '{}.jpg'. If you already have a file called '{}.jpg', this will probably overwrite your original file."

HOW_TO_NAME_FILENAME = "To specify a unique filename, you can write filename='mycustomfilename' in the arguments of your method call. For example: ui.save_collage(filename='mycustomfilename') will save a file called 'mycustomfilename.jpg'"

CANCEL_OPTION = "If you'd like to cancel this operation and knowing what you know now, type 'c' and then press enter. Otherwise, just press enter."

WHITESPACE = "\n"

CANCEL_CONFIRMATION = "You've requested to cancel, so the method you called will not be completed as dialed."

PROMPT_FOR_USERNAME = "Please enter your instagram username here and then press enter:\n\n"

PROMPT_FOR_PASSWORD = "Please enter your instagram password below and then press enter (don't worry that the cursor doesn't move while you type):\n\n"

SET_POSITIVE_SPACE_CONFIRMATION = "You have updated the positive space in your collage."

SET_MAIN_IMAGE_CONFIRMATION = "You have updated the main image in your collage."

SET_MOON_PHASE_CONFIRMATION = "You have set a moonphase for the date: "

SET_NEGATIVE_SPACE_CONFIRMATION = "You have updated the negative space in your collage."

SIGNING_INTO_INSTAGRAM_WARNING = "Some notes about this: In how this package is currently set up, you need to type your username and password in the command line to log into Instagram. Do this at your own risk... As a general rule of thumb, it's obviously not good to type your passwords just anywhere. If you want a more secure way of getting your photo to instagram, just email the file to yourself to get it to your phone."

INSTAGRAM_BOT_WARNING = "Another note: This package uses a python package called InstagramAPI that isn't officially approved by Instagram. Using it may violate terms of service with Instagram, and you might run into some issues 'verifying' your login."

INSTAGRAM_BRAND_GROWTH_RAMBLE= "A last note: Instagram doesn't like 3rd party API packages like this, since they're often used to automate accounts to create brand growth (and because bots can be annoying when used in certain ways). By using this InstagramAPI package, you *may* be violating some Instagram rules."

SAVING_INSTAGRAM_SESSION_MESSAGE = "Would you like to save your instagram login for next time? This means that your instagram information (including your username and password and the 'object' that is your instagram account information), will be saved so that you don't need to login next time."

PROMPT_FOR_INSTAGRAM_CAPTION = "If you would like to include a caption with this post, please type it below and press enter. If you don't want one, just type nothing and press enter."

INSTAGRAM_POST_FINAL_CHECK = "OK, you're about to try to post your collage to instagram. You'll probably see some messages from the InstagramAPI package here (like 429 and 404 errors). You can probably ignore those, but if something ends up not working, they could be clues as to why the post didn't work.\n\nIf you'd like to go ahead and try to post your image to instagram, type 'y' and press enter. Otherwise, just press enter to cancel.\n\n"

INSTAGRAM_POST_ATTEMPT_CONFIRMATION = "If you didn't see any major errors, then your picture must have posted to instagram! Check you account. If there *were* issues, don't worry too much about it and just email your picture to yourself if you'd like to post it somewhere."

SKIP_TEXT_MATCH = "pass"



from PIL import Image
import urllib.request
import io

class CustomImage():
    def __init__(self, img_size):
        self.size = img_size
        return

    def set_image(self, url="", instagram_url="", filename="", color=""):
        #if more than one of these parameters are provided, raise an error.

        if url: return self.load_from_url(url)
        if instagram_url: return self.load_from_instagram(instagram_url)
        if filename: return self.load_from_filesystem(filename)
        if color: return self.make_custom_image(color)

    def make_custom_image(self, color):
        print("making an image that has the color" + str(color))
        self.image = Image.new('RGB', self.size, color)
        return self.image

    def load_from_instagram(self, instagram_url):
        #TODO check if link has extra info at the end
        #https://nono.ma/says/get-an-instagram-image-url
        try:
            self.load_from_url(instagram_url + "media/?size=l")
        except:
            raise Exception("I can't seem to download that image from that link: " + instagram_url + ". Check that it's an instagram link in the form 'https://instagram.com/p/SOME_NUMBERS_AND_LETTERS/'.")
        return self.image

    def load_from_filesystem(self, path):
        self.image = Image.open(path)
        self.resize_image()
        return self.image

    def load_from_url(self, url):
        print("Requesting image from the internets...")
        print(" ,-,\n/.(\n\ {\n `-`")
        fd = urllib.request.urlopen(url)
        image_file = io.BytesIO(fd.read())
        self.image = Image.open(image_file)
        self.resize_image()
        print("Got it!")
        return self.image

    def resize_image(self):
        self.image.resize(self.size, Image.ANTIALIAS)



from moonmask.collage import Collage
from moonmask.instagram_wrapper import InstagramWrapper
from moonmask.res.constants import *
from moonmask.res.ascii_art import *
from moonmask.res.ui_text import *
import getpass
import os
from time import sleep
import textwrap
import shutil

class MoonMaskUI():

    def __init__(self):
        self.collage = Collage()
        self.instagram = None
        self.moonmask_filename = None
        self.terminal_width = shutil.get_terminal_size().columns
        self.text_width = int(self.terminal_width * 0.75)
        self.text_margin = int(self.terminal_width * 0.12)
        self.text_wrapper = textwrap.TextWrapper(width=self.text_width)
        self.margin_symbols = "|"*(self.text_margin - 2)+"  "
        self.command_prompt = ">"*3+"  "

    def set_moon_phase(self, date=None, relative_date="today", filename=None):
        """Sets the moon phase that will be used as a mask on the image

        Keyword arguments:
        date -- the date in format YYYYMMDD
        relative_date -- defaults to "today". Other accepted options are "yesterday" and "tomorrow"
        """
        self.print_text((WHITESPACE))
        self.collage.set_mask(date, relative_date, filename)
        moon_phase_date = str(self.collage.get_moon_phase_date())
        self.print_text((WHITESPACE))
        self.print_text((SET_MOON_PHASE_CONFIRMATION))
        self.print_text((WHITESPACE))
        self.print_text((moon_phase_date))

    def set_main_image(self, url="", instagram_url="", filename="", color=""):
        self.print_text((WHITESPACE))
        self.collage.set_main_image(url, instagram_url, filename, color)
        self.print_text((WHITESPACE))
        self.print_text((SET_MAIN_IMAGE_CONFIRMATION))

    def set_negative_space(self, url="", instagram_url="", filename="", color=""):
        self.print_text((WHITESPACE))
        self.collage.set_mask_negative_space(url, instagram_url, filename, color)
        self.print_text((WHITESPACE))
        self.print_text((SET_NEGATIVE_SPACE_CONFIRMATION))

    def set_positive_space(self, url="", instagram_url="", filename="", color=""):
        self.print_text((WHITESPACE))
        self.collage.set_mask_positive_space(url, instagram_url, filename, color)
        self.print_text((WHITESPACE))
        self.print_text((SET_POSITIVE_SPACE_CONFIRMATION))

    def save_collage(self, filename="moonmask", main_image_file=None, mask_file=None, positive_space_file=None, negative_space_file=None, positive_space_transparency=200, negative_space_transparency=50, dimensionality=3, img_size=1000):
        self.print_text((WHITESPACE))
        if (filename.lower() == "moonmask"):
            self.print_text((OVERWRITE_FILENAME_NOTICE.format("moonmask", "moonmask")))
            self.print_text((WHITESPACE))
            self.print_text((HOW_TO_NAME_FILENAME))
            self.print_text((WHITESPACE))
            user_decision = self.get_input((CANCEL_OPTION))
            if (user_decision.strip().lower() == "c"):
                return
        self.collage.make_collage(filename, main_image_file, mask_file, positive_space_file, negative_space_file, positive_space_transparency, negative_space_transparency, dimensionality, img_size)
        self.moonmask_filename = filename + ".jpg"
        self.print_text((WHITESPACE))
        self.print_text((SAVE_COLLAGE_CONFIRMATION))
        self.print_text((WHITESPACE))
        self.print_text((self.moonmask_filename))

    def post_to_instagram(self, save_session=False, usertags=[]):
        self.print_text((WHITESPACE))
        self.print_text((SIGNING_INTO_INSTAGRAM_WARNING))
        self.enter_to_continue()
        self.print_text((WHITESPACE))
        self.print_text((INSTAGRAM_BOT_WARNING))
        self.enter_to_continue()
        self.print_text((INSTAGRAM_BRAND_GROWTH_RAMBLE))
        self.enter_to_continue()
        self.print_text((WHITESPACE))
        user_decision = self.get_input((CANCEL_OPTION))
        if (user_decision.strip().lower() == "c"):
            self.print_text((CANCEL_CONFIRMATION))
            self.print_text((WHITESPACE))
            return
        self.print_text((WHITESPACE))
        username = self.get_input((PROMPT_FOR_USERNAME))
        self.print_text((WHITESPACE))
        password = getpass.getpass(self.text_format(PROMPT_FOR_PASSWORD))
        self.print_text((WHITESPACE))
        save_session = self.get_input((SAVING_INSTAGRAM_SESSION_MESSAGE))
        self.print_text((WHITESPACE))

        if save_session.strip().lower() == "y": save_session = True

        caption = self.get_input((PROMPT_FOR_INSTAGRAM_CAPTION))
        self.print_text((WHITESPACE))

        user_decision = self.get_input((INSTAGRAM_POST_FINAL_CHECK))
        if user_decision.strip().lower() == "y":
            self.instagram = InstagramWrapper(username, password, save_session)
            self.instagram.post_image(self.moonmask_filename, caption, usertags=usertags)
        else:
            self.print_text((CANCEL_CONFIRMATION))
            return
        self.print_text((INSTAGRAM_POST_ATTEMPT_CONFIRMATION))


    def show_info(self):
        self.clear_screen()
        self.print_text((WHITESPACE))
        self.print_ascii_art(MOON_WAXING_ASCII_ART)
        self.print_ascii_art(TITLE_ASCII_ART)
        self.print_text((SHOW_INFO))
        self.print_text((WHITESPACE))
        self.print_text(THIS_IS_A_TUTORIAL_MESSAGE)
        self.print_ascii_art(MOON_WANING_ASCII_ART)
        self.print_text((WHITESPACE))
        self.enter_to_continue()

    def show_credits(self):
        self.clear_screen()
        self.print_ascii_art(MOON_WAXING_ASCII_ART)
        self.print_ascii_art(CREDITS_ASCII_ART)
        self.print_list(CREDITS_LIST)
        self.print_ascii_art(MOON_WANING_ASCII_ART)
        self.enter_to_continue()

    def show_commands(self):
        self.clear_screen()
        self.print_ascii_art(MOON_WAXING_ASCII_ART)
        self.print_ascii_art(ASCII_ART_NUMBERS[0])
        self.print_multiline_text(PURPOSES_OF_PACKAGE_TUTORIAL_STEP)
        self.enter_to_continue()
        self.clear_screen()
        self.print_ascii_art(MOON_WANING_ASCII_ART)

        self.print_ascii_art(ASCII_ART_NUMBERS[1])
        self.print_multiline_text(IMPORT_CLASS_TUTORIAL_STEP, substitute_text = IMPORT_UI_COMMAND)
        self.ask_user_to_repeat_command(IMPORT_UI_COMMAND)
        self.enter_to_continue()
        self.clear_screen()
        self.print_ascii_art(MOON_WAXING_ASCII_ART)

        self.print_ascii_art(ASCII_ART_NUMBERS[2])
        self.print_multiline_text(INITIALIZE_UI_TUTORIAL_STEP, substitute_text=INITIALIZE_UI_COMMAND)
        self.ask_user_to_repeat_command(INITIALIZE_UI_COMMAND)
        self.enter_to_continue()
        self.clear_screen()
        self.print_ascii_art(MOON_WANING_ASCII_ART)

        self.print_ascii_art(ASCII_ART_NUMBERS[3])
        self.print_multiline_text(SET_MOON_PHASE_TUTORIAL_STEP, SET_MOON_PHASE_COMMAND)
        self.ask_user_to_repeat_command(SET_MOON_PHASE_COMMAND)
        self.set_moon_phase()
        self.enter_to_continue()
        self.clear_screen()

        self.print_ascii_art(ASCII_ART_NUMBERS[4])
        self.print_multiline_text(INTRO_SAVE_COLLAGE_TUTORIAL_STEP, SAVE_COLLAGE_COMMAND)
        self.ask_user_to_repeat_command(SAVE_COLLAGE_COMMAND)
        self.save_collage(filename='moon')
        self.enter_to_continue()
        self.clear_screen()

        self.print_ascii_art(ASCII_ART_NUMBERS[5])
        self.print_ascii_art(DIAGRAM_ASCII_ART_POSITIVE_SPACE)
        self.print_multiline_text(SET_POSITIVE_SPACE_TUTORIAL_STEP, SET_POSITIVE_SPACE_COMMAND)
        self.ask_user_to_repeat_command(SET_POSITIVE_SPACE_COMMAND)
        self.set_positive_space(color='yellow')
        self.enter_to_continue()
        self.clear_screen()

        self.print_ascii_art(ASCII_ART_NUMBERS[6])
        self.print_multiline_text(LETS_SAVE_TUTORIAL_STEP, SAVE_COLLAGE_COMMAND)
        self.ask_user_to_repeat_command(SAVE_COLLAGE_COMMAND)
        self.save_collage(filename='moon')
        self.enter_to_continue()
        self.clear_screen()

        #set negative space to yellow and save it
        self.print_ascii_art(ASCII_ART_NUMBERS[7])
        self.print_ascii_art(DIAGRAM_ASCII_ART_NEGATIVE_SPACE)
        self.print_multiline_text(SET_NEGATIVE_SPACE_TUTORIAL_STEP, SET_NEGATIVE_SPACE_COMMAND)
        self.ask_user_to_repeat_command(SET_NEGATIVE_SPACE_COMMAND)
        self.set_negative_space(color='yellow')
        self.enter_to_continue()
        self.clear_screen()

        self.print_ascii_art(ASCII_ART_NUMBERS[8])
        self.print_multiline_text(LETS_SAVE_TUTORIAL_STEP, SAVE_COLLAGE_COMMAND)
        self.ask_user_to_repeat_command(SAVE_COLLAGE_COMMAND)
        self.save_collage(filename='moon')
        self.enter_to_continue()
        self.clear_screen()

        #set main image to color light coral and save it
        self.print_ascii_art(ASCII_ART_NUMBERS[9])
        self.print_ascii_art(DIAGRAM_ASCII_ART_COMPLETE)
        self.print_multiline_text(SET_MAIN_IMAGE_TUTORIAL_STEP, SET_MAIN_IMAGE_COMMAND)
        self.ask_user_to_repeat_command(SET_MAIN_IMAGE_COMMAND)
        self.set_main_image(color=(240,128,128))
        self.enter_to_continue()
        self.clear_screen()

        #save it
        self.print_ascii_art(ASCII_ART_NUMBERS[10])
        self.print_multiline_text(LETS_SAVE_TUTORIAL_STEP, SAVE_COLLAGE_COMMAND)
        self.ask_user_to_repeat_command(SAVE_COLLAGE_COMMAND)
        self.save_collage(filename='moon')
        self.enter_to_continue()
        self.clear_screen()

        #more info
        self.print_ascii_art(ASCII_ART_NUMBERS[11])
        self.print_multiline_text(MORE_INFO, SET_POSITIVE_SPACE_COMMAND)
        self.enter_to_continue()
        self.clear_screen()

    def goodbye_message(self):
        self.print_ascii_art(MOON_WAXING_ASCII_ART)
        self.print_text(FINISHED_TUTORIAL_MESSAGE)
        self.print_text((WHITESPACE))
        self.print_ascii_art(MOON_WANING_ASCII_ART)
        self.enter_to_continue()
        self.clear_screen()
        self.show_credits()

    def show_tutorial(self):
        self.show_info()
        self.print_text(NAVIGATION_REMINDER)
        self.enter_to_continue()
        self.show_commands()
        self.goodbye_message()

    def print_ascii_art(self, ascii_art_list):
        for element in ascii_art_list:
            self.print_text(element[:self.text_width])
        print("")

    def clear_screen(self):
        os.system('cls' if os.name == 'nt' else 'clear')
        self.mini_sleep()

    def mini_sleep(self):
        sleep(0.2)

    def enter_to_continue(self):
        self.print_text((WHITESPACE))
        self.get_input((PRESS_ENTER_TO_CONTINUE))
        self.print_text((WHITESPACE))

    def print_list(self, list_to_print):
        for element in list_to_print:
            self.print_text((" * " + element))
            print("")

    def print_text(self, text):
        print(self.text_format(text))

    def print_multiline_text(self, text_list, substitute_text=""):
        for text in text_list:
            self.print_text(text.format(self.command_prompt + substitute_text))
            print("")

    def get_input(self, prompt, command_prompt=False):
        if True:
            return input(self.text_format(prompt)+"\n\n" + self.margin_symbols + self.command_prompt)
        else:
            return input(self.text_format(prompt)+"\n\n"+self.margin_symbols)

    def ask_user_to_repeat_command(self, command):
        user_input = self.get_input(TUTORIAL_PROMPT_FOR_COMMAND, command_prompt=True)
        while user_input != command and user_input != SKIP_TEXT_MATCH:
            self.print_text(WHITESPACE)
            user_input = self.get_input(TUTORIAL_TRY_AGAIN_PROMPT.format(command, command_prompt=True))
        self.print_text(WHITESPACE)
        self.print_text(USER_SUCCESS_MESSAGE)

    def text_format(self,text):
        return textwrap.indent(text=self.text_wrapper.fill(text=text), prefix=self.margin_symbols)




import moonmask.custom_image as custom_image
from moonmask.res.constants import *
from datetime import datetime, timezone, timedelta

class MoonImage(custom_image.CustomImage):
    def __init__(self, size):
        self.size = size
        super()
        return

    def set_moon_image(self, relative_date="today", date=None, filename=None):
        """Sets the image that will be used as a mask on the image

        Keyword arguments:
        date -- the date in format YYYYMMDD
        relative_date -- defaults to "today". Other accepted options are "yesterday" and "tomorrow"
        """

        #if more than one of these parameters are provided, raise an error.
        #TODO check for a cleaner way to do this
        url = ""

        if (filename == None):
            if (date):
                self.datetime = datetime.strptime(date, '%Y-%m-%d').replace(tzinfo=timezone.utc)
            elif (relative_date.lower() == "today"):
                self.datetime = datetime.now(timezone.utc)
            elif (relative_date.lower() == "tomorrow"):
                self.datetime = datetime.now(timezone.utc) + timedelta(days=1)
            elif (relative_date.lower() == "yesterday"):
                self.datetime = datetime.now(timezone.utc) + timedelta(days=-1)
            self.frame_id = self.get_nasa_frame_id(date)
            url = self.get_moon_url(date)

        self.image = self.set_image(url=url, filename=filename)
        return self.image

    def get_nasa_frame_id(self, date):
        #code logic courtesy of Ernie Wright
        year = self.datetime.year
        if (year != 2019):
            moon_imagenum = 1
        janone = datetime(year, 1, 1, 0, 0, 0, 0, tzinfo=timezone.utc );
        moon_imagenum = int(round((self.datetime - janone ).total_seconds() / 3600))
        if (moon_imagenum > 8760):
            moon_imagenum = 8760
        return str(moon_imagenum).zfill(4)

    def get_moon_url(self, date):
        self.nasa_id = NASA_ID["2019"]
        url = "https://svs.gsfc.nasa.gov/vis/a000000/a00{year_id_modulo}/a00{year_id}/frames/730x730_1x1_30p/moon.{frame_id}.jpg".format(
            year_id_modulo = str(self.nasa_id - self.nasa_id % 100),
            year_id = str(self.nasa_id),
            frame_id = str(self.frame_id)
        )
        return url

    def save_moon(self):
        if (self.datetime and self.image):
            self.image.save("moon-image-" + self.datetime + ".jpg")

    def get_moon_phase_date(self):
        return self.datetime



from moonmask.custom_instagram_api import CustomInstagramAPI
import requests
from io import BytesIO
from moonmask.res.constants import *
import pickle

class InstagramWrapper():

    def __init__(self, username, password, save_session=False):
        try:
            self.recover_saved_session(username, password)
        except:
            print("using a new instagram session")
            self.ig = CustomInstagramAPI(username, password)

        self.ig.login()
        self.ig.generateDeviceId("0")
        if save_session:
            try:
                with open(SESSIONS_PATH, 'wb') as fp:
                    pickle.dump(self.ig, fp)
                fp.close()
            except:
                raise Exception("Not able to dump current session into " + SESSIONS_PATH)

    def recover_saved_session(self, username, password):
        print("recovering saved session...")
        with open(SESSIONS_PATH, 'rb') as fp:
            self.ig = pickle.load(fp)
            fp.close()

    def post_image(self, image_path, caption, usertags=USERTAGS, test=False):
        self.ig.login()
        self.ig.generateDeviceId("0")
        photo_path = image_path
        self.ig.uploadPhoto(image_path, caption=caption, usertags=USERTAGS)

    def get_image_info(self, post_url):
        self.ig.login()
        info = {}
        info['media_id'] = self.get_media_id(post_url)
        media_info = self.ig.LastJson
        info['url'] = media_info['items'][0]['image_versions2']['candidates'][0]['url']
        info['username'] = media_info['items'][0]['user']['username']
        info['user_id'] = media_info['items'][0]['user']['pk']
        return info

    def get_media_id(self, url):
        req = requests.get('https://api.instagram.com/oembed/?url={}'.format(url))
        media_id = req.json()['media_id']
        return media_id

    def get_username(self, media_id):
        if not media_id:
            return None
        self.ig.login()
        self.ig.mediaInfo(media_id)
        return self.ig.LastJson['items'][0]['user']['username']



from PIL import Image

class Mask():
    def __init__(self, image, alpha_threshold, img_size, transparency=0):
        self.alpha_threshold = alpha_threshold
        self.transparency = transparency
        self.image = image
        self.image = self.image.resize(img_size, Image.ANTIALIAS)
        self.make_mask()
        return

    def make_mask(self):
        self.image = self.image.point(lambda p: p > self.alpha_threshold and 255)
        if (self.transparency != 0):
            self.image = self.image.point(lambda p: self.transparency)
        self.image = self.image.convert("L")
        return

    def get_mask(self):
        return self.image



from distutils.core import setup
from setuptools import setup, find_packages

setup(
    name = 'moonmask',
    packages = find_packages(),
    version = '1.7',
    license='MIT',
    description = 'Gets moon visualizations courtesy of Ernie Wright and uses them for artistic collages',
    author = 'Sadie Parker',
    author_email = 'sadiemparker@gmail.com',
    url = 'https://github.com/spacerest/moonmask',
    download_url = 'https://github.com/spacerest/moonmask/archive/v_17.tar.gz',
    keywords = ['MOON', 'ART'],
    install_requires=[
        'Pillow==5.4.1',
        'decorator==4.0.11',
        'imageio==2.1.2',
        'InstagramAPI==1.0.2',
        'moviepy==0.2.3.2',
        'numpy==1.16.2',
        'requests==2.20.0',
        'requests-toolbelt==0.7.0',
        'tqdm==4.11.2'
    ],
    classifiers=[
    'Development Status :: 4 - Beta',
    'Intended Audience :: Other Audience',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
  ],
)



from InstagramAPI import InstagramAPI
import time
from requests_toolbelt import MultipartEncoder
import json
from PIL import Image

class CustomInstagramAPI(InstagramAPI):

    def __init__(self, *args, **kwargs):
        print("hi, initting custom instagram api")
        super().__init__(*args, **kwargs)

    def configure(self, upload_id, photo, caption='', usertags=[]):
        usertags = {'in': [{'user_id': str(u['user_id']), 'position': u['position']} for u in usertags]}
        usertags = json.dumps(usertags, separators=(',', ':'))
        (w, h) = Image.open(photo).size
        data = json.dumps({'_csrftoken': self.token,
                           'media_folder': 'Instagram',
                           'source_type': 4,
                           '_uid': self.username_id,
                           '_uuid': self.uuid,
                           'caption': caption,
                           'upload_id': upload_id,
                           'device': self.DEVICE_SETTINTS,
                           'usertags': usertags,
                           'edits': {
                               'crop_original_size': [w * 1.0, h * 1.0],
                               'crop_center': [0.0, 0.0],
                               'crop_zoom': 1.0
                           },
                           'extra': {
                               'source_width': w,
                               'source_height': h
                           }})
        return self.SendRequest('media/configure/?', self.generateSignature(data))

    def uploadPhoto(self, photo, caption=None, upload_id=None, is_sidecar=None, usertags=[]):
        """
        Uploads a single photo
        Args:
            photo
            caption (optional)
            upload_id (optional)
            is_sidecar (optional)
            usertags (optional)
                needs to be a list of dicts like:
                    [{"user_id": 123456789, "position": [x, y]},
                     ....
                    ]
                where each item represents a usertag and x, y are floats between 0.0 and 1.0
        """
        if upload_id is None:
            upload_id = str(int(time.time() * 1000))
        data = {'upload_id': upload_id,
                '_uuid': self.uuid,
                '_csrftoken': self.token,
                'image_compression': '{"lib_name":"jt","lib_version":"1.3.0","quality":"87"}',
                'photo': ('pending_media_%s.jpg' % upload_id, open(photo, 'rb'), 'application/octet-stream', {'Content-Transfer-Encoding': 'binary'})}
        if is_sidecar:
            data['is_sidecar'] = '1'

        # If usertags are provided, verify that the entries are valid.
        if usertags:
            self.throwIfInvalidUsertags(usertags)

        m = MultipartEncoder(data, boundary=self.uuid)
        self.s.headers.update({'X-IG-Capabilities': '3Q4=',
                               'X-IG-Connection-Type': 'WIFI',
                               'Cookie2': '$Version=1',
                               'Accept-Language': 'en-US',
                               'Accept-Encoding': 'gzip, deflate',
                               'Content-type': m.content_type,
                               'Connection': 'close',
                               'User-Agent': self.USER_AGENT})
        response = self.s.post(self.API_URL + "upload/photo/", data=m.to_string())
        if response.status_code == 200:
            if self.configure(upload_id, photo, caption, usertags):
                self.expose()
        return False

    def throwIfInvalidUsertags(self, usertags):
        for user_position in usertags:
            # Verify this usertag entry, ensuring that the entry is format
            # ['position'=>[0.0,1.0],'user_id'=>'123'] and nothing else.
            correct = True
            if isinstance(user_position, dict):
                position = user_position.get('position', None)
                user_id = user_position.get('user_id', None)

                if isinstance(position, list) and len(position) == 2:
                    try:
                        x = float(position[0])
                        y = float(position[1])
                        if x < 0.0 or x > 1.0:
                            correct = False
                        if y < 0.0 or y > 1.0:
                            correct = False
                    except:
                        correct = False
                try:
                    user_id = int(user_id)
                    if user_id < 0:
                        correct = False
                except:
                    correct = False
            if not correct:
                raise Exception('Invalid user entry in usertags array.')
